import CanvasContainer from '../references/canvas-container.js';
import BackgroundCanvas from './background-canvas.js';
import ForegroundCanvas from './foreground-canvas.js';

import './project.scss';


class GanttChart {
  Options = null;
  CanvasContainer = null;

  #debug = false;
  Tasks = [];
  StartDate = null;
  Duration = 30;
  StartOfWeek = 1;


  constructor(el, options) {
    const a = this;

    a.Options = Object.assign(a.DefaultOptions, options);
    a.CanvasContainer = new CanvasContainer(el);

    a.#initialiseComponents();
  }

  #initialiseComponents() {
    const a = this;

    // Add background canvas layer
    const layer1 = a.CanvasContainer.AddLayer();

    const backCanvasLayer = new BackgroundCanvas(layer1);
    backCanvasLayer.Options = a.Options;

    a.CanvasContainer.Layer.push(backCanvasLayer);

    // Add foreground canvas layer
    const layer2 = a.CanvasContainer.AddLayer();

    const foreCanvasLayer = new ForegroundCanvas(layer2);
    foreCanvasLayer.Options = a.Options;

    a.CanvasContainer.Layer.push(foreCanvasLayer);

    a.CanvasContainer.Invalidate();

    Document.removeClass(a.CanvasContainer.FlowContainer, "border");
    Document.addClass(a.CanvasContainer.FlowContainer, "gantt-chart");

  }


  get DefaultOptions() {
    return {
      DayWidth: 24,
      HeaderRow: {
        Height: [ 20, 20 ]
      },
      Row: {
        Height: 28,
        BackColour: "rgba(235, 235, 235, 0.3)",
        Task: {
          Height: 14,
          BorderColour: "#555555",
          FillColour: "#D8EEDB"
        },
        OrphanTask: {
          Height: 14,
          BorderColour: "#555555",
          FillColour: "#9CC2E6"
        },
        ChildTask: {
          Height: 14,
          BorderColour: "#555555",
          FillColour: "#FFF5C1"
        },
        CollatedTask: {
          Height: 6,
          BorderColour: "#555555",
          FillColour: "#555555"
        }
      },
      Column: {
        SatColour: "rgba(233, 237, 239, 0.8)",
        SunColour: "rgba(233, 237, 239, 0.9)",
      },
      Line: {
        Colour: "#555555",
        Width: 1,
        ArrowSize: 5
      },
      DateFont: "7pt sans-serif",
      DateForeColour: "#636363",
      BorderWidth: 1,
      BorderColour: "#B8B8B8",
      BorderDashPattern: [1, 1],
      MinimumRowCount: 6,
      ShowDateLines: true,
      ShowStartDayOfWeekLine: true,
      ShowRowLines: true,
      ShowRowStripes: true
    };
  }

  get HeaderHeight() {
    const a = this;

    return a.Options.HeaderRow.Height[0] + a.Options.HeaderRow.Height[1];
  }

  get Debug() {
    return this.#debug;
  }

  set Debug(value) {
    const a = this;

    a.#debug = value;

    a.CanvasContainer.Layer.forEach(e => {
      if (typeof(e.Debug) == "undefined") {
        return;
      }

      e.Debug = value;
    });

  }


  Clear() {
    const a = this;

    a.CanvasContainer.Clear();

    a.StartDate = new Date();
    a.Duration = 30;
    a.StartOfWeek = 1;    // 1 = Monday
    a.Tasks = [];

    // Invalidate canvas (background, foreground)
    a.CanvasContainer.Layer[0].Load(new Date(), a.Duration, a.StartOfWeek, a.Tasks.length);
    a.CanvasContainer.Layer[1].Load(new Date(), a.Tasks);

    a.Invalidate();
  }

  Invalidate() {
    const a = this;

    a.CanvasContainer.Clear();

    const width = ((a.Duration + 2) * a.Options.DayWidth);
    const height = (Math.max(a.Tasks.length, a.Options.MinimumRowCount) * a.Options.Row.Height) + a.HeaderHeight;

    a.CanvasContainer.Size = { W: width, H: height };

    a.CanvasContainer.Invalidate();
  }

  Load(project) {
    const a = this;

    a.CanvasContainer.Clear();

    if (project == null) {
      return;
    }

    a.StartDate = new Date(project.StartDate);
    a.Duration = project.Duration;
    a.StartOfWeek = project.Project.StartOfWeek;
    a.Tasks = project.ExportTasks();

    // Invalidate canvas (background, foreground)
    a.CanvasContainer.Layer[0].Load(new Date(project.StartDate), project.Duration, project.Project.StartOfWeek, a.Tasks.length);
    a.CanvasContainer.Layer[1].Load(new Date(project.StartDate), a.Tasks);

    a.Invalidate();
  }

}


export default GanttChart;